@@ -2,7 +2,7 @@
 /*
  * FCRON - periodic command scheduler 
  *
- *  Copyright 2000-2016 Thibault Godouet <fcron@free.fr>
+ *  Copyright 2000-2021 Thibault Godouet <fcron@free.fr>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -453,7 +453,7 @@ read_file(const char *file_name, cf_t * cf, int is_system_startup)
     int has_read_cl_first = 0;  /* have we read S_FIRST_T? */
 #ifdef WITH_SELINUX
     int flask_enabled = is_selinux_enabled();
-    int retval;
+    int retval = -1;
     struct av_decision avd;
     char *user_name = NULL;
 #endif
@@ -540,19 +540,34 @@ read_file(const char *file_name, cf_t * cf, int is_system_startup)
         if (get_default_context(user_name, NULL, &cf->cf_user_context))
             error_e("NO CONTEXT for Linux user '%s' (SELinux user '%s')",
                     cf->cf_user, user_name);
-        retval =
-            security_compute_av(cf->cf_user_context, cf->cf_file_context,
-                                SECCLASS_FILE, FILE__ENTRYPOINT, &avd);
 
-        if (retval || ((FILE__ENTRYPOINT & avd.allowed) != FILE__ENTRYPOINT)) {
+        /* we no longer need those - clean them up */
+        Free_safe(sename);
+        Free_safe(selevl);
+
+        security_class_t sec_class = string_to_security_class("file");
+        if (!sec_class) {
+            error_e("Failed to translate security class 'file'\n");
+            goto err;
+        }
+
+        access_vector_t access_vec = string_to_av_perm(sec_class, "entrypoint");
+        if (!access_vec) {
+            error_e("Failed to translate security class file\n");
+            goto err;
+        }
+
+        /* if we get here, sec_class and access_vec are both defined */
+        retval = security_compute_av(cf->cf_user_context, cf->cf_file_context,
+                                sec_class, access_vec, &avd);
+
+        if (retval || ((access_vec & avd.allowed) != access_vec)) {
             syslog(LOG_ERR, "ENTRYPOINT FAILED for Linux user '%s' "
                    "(CONTEXT %s) for file CONTEXT %s", cf->cf_user,
                    cf->cf_user_context, cf->cf_file_context);
             goto err;
         }
 
-        Free_safe(sename);
-        Free_safe(selevl);
     }
 #endif
 
@@ -816,7 +831,7 @@ add_line_to_file(cl_t * cl, cf_t * cf, uid_t runas, char *runas_str,
                  time_t t_save, int is_system_startup)
     /* check if the line is valid, and if yes, add it to the file cf */
 {
-    time_t slept = now - t_save;
+    time_t sleep_duration = now - t_save;
 
     if (cl->cl_shell == NULL || cl->cl_runas == NULL || cl->cl_mailto == NULL) {
         error("Line is not valid (empty shell, runas or mailto field)"
@@ -853,173 +868,16 @@ add_line_to_file(cl_t * cl, cf_t * cf, uid_t runas, char *runas_str,
         Set(cl->cl_mailto, cl->cl_file->cf_user);
     }
 
-    /* job has been stopped during execution: insert it in lavg or serial queue
-     * if it was in one at fcron's stops.  */
-    /* NOTE: runatreboot is prioritary over jobs that were still running
-     * when fcron stops, because the former will get run quicker as they are not
-     * put into the serial queue. runatreboot jobs will be handled later on. */
-    if (cl->cl_numexe > 0 && !is_runatreboot(cl->cl_option)) {
-
-        cl->cl_numexe = 0;
-        if (is_lavg(cl->cl_option)) {
-            if (!is_strict(cl->cl_option))
-                add_lavg_job(cl, -1);
-        }
-        else if (is_serial(cl->cl_option)
-                 || is_serial_once(cl->cl_option))
-            add_serial_job(cl, -1);
-        else {
-            /* job has been stopped during execution :
-             * launch it again */
-            warn("job '%s' did not finish : running it again.", cl->cl_shell);
-            set_serial_once(cl->cl_option);
-            add_serial_job(cl, -1);
-        }
+    /* make sure the timefreq is valid on @-lines or we could end up with
+     * infinite loops */
+    if (!is_td(cl->cl_option) && cl->cl_timefreq < 1) {
+        error("Invalid timefreq %ld for job '%s': setting to 1 day",
+              cl->cl_timefreq, cl->cl_shell);
+        cl->cl_timefreq = 3600 * 24;
     }
 
-    if (is_system_startup || is_volatile(cl->cl_option)) {
-        clear_hasrun(cl->cl_option);
-    }
-
-    if (is_runonce(cl->cl_option) && is_hasrun(cl->cl_option)) {
-        /* if we get here, then is_system_startup is_volatile are both false */
-        /* do nothing: don't re-schedule or add to the job queue */
-        explain("job '%s' with runonce set has already run since last "
-                "system startup: not re-scheduling.", cl->cl_shell);
-    }
-    else if (is_td(cl->cl_option)) {
-
-        /* set the time and date of the next execution  */
-        if (is_system_startup && is_runatreboot(cl->cl_option)) {
-
-            if (is_notice_notrun(cl->cl_option)) {
-
-                if (cl->cl_runfreq == 1) {
-                    /* %-line */
-                    set_next_exe_notrun(cl, SYSDOWN_RUNATREBOOT);
-                }
-                else {
-                    /* set next exe and mail user */
-                    time_t since = cl->cl_nextexe;
-
-                    cl->cl_nextexe = now;
-                    mail_notrun_time_t(cl, SYSDOWN, since);
-                }
-
-            }
-            else {
-                cl->cl_nextexe = now;
-            }
-
-            insert_nextexe(cl);
-
-        }
-        else if (cl->cl_nextexe <= now) {
-            if (cl->cl_nextexe == 0)
-                /* the is a line from a new file */
-                set_next_exe(cl, NO_GOTO, -1);
-            else if (cl->cl_runfreq == 1 && is_notice_notrun(cl->cl_option))
-                set_next_exe_notrun(cl, SYSDOWN);
-            else if (is_bootrun(cl->cl_option) && t_save != 0
-                     && cl->cl_runfreq != 1) {
-                if (cl->cl_remain > 0 && --cl->cl_remain > 0) {
-                    debug("    cl_remain: %d", cl->cl_remain);
-                }
-                else {
-                    /* run bootrun jobs */
-                    cl->cl_remain = cl->cl_runfreq;
-                    debug("   boot-run '%s'", cl->cl_shell);
-                    if (!is_lavg(cl->cl_option)) {
-                        set_serial_once(cl->cl_option);
-                        add_serial_job(cl, -1);
-                    }
-                    else
-                        add_lavg_job(cl, -1);
-                }
-                set_next_exe(cl, STD, -1);
-            }
-            else {
-                if (is_notice_notrun(cl->cl_option)) {
-                    /* set next exe and mail user */
-                    time_t since = cl->cl_nextexe;
-
-                    set_next_exe(cl, NO_GOTO, -1);
-                    mail_notrun_time_t(cl, SYSDOWN, since);
-
-                }
-                else
-                    set_next_exe(cl, NO_GOTO, -1);
-            }
-        }
-        else {
-            /* value of nextexe is valid : just insert line in queue */
-            insert_nextexe(cl);
-        }
-    }
-    else {                      /* is_td(cl->cl_option) */
-        if (cl->cl_timefreq < 1) {
-            error("Invalid timefreq %ld for job '%s': setting to 1 day",
-                  cl->cl_timefreq, cl->cl_shell);
-            cl->cl_timefreq = 3600 * 24;
-        }
-
-        /* standard @-lines */
-        if (is_system_startup && is_runatreboot(cl->cl_option)) {
-            cl->cl_nextexe = now;
-        }
-        /* t_save == 0 means this is a new file, hence a new line */
-        else if (t_save == 0 || is_volatile(cl->cl_option)
-                 || (is_system_startup && (is_rebootreset(cl->cl_option)
-                                           || is_runonce(cl->cl_option)))) {
-            /* cl_first is always saved to disk for a volatile line */
-            if (cl->cl_first == LONG_MAX) {
-                cl->cl_nextexe = TIME_T_MAX;
-            }
-            else {
-                cl->cl_nextexe = now + cl->cl_first;
-                if (cl->cl_nextexe < now || cl->cl_nextexe > TIME_T_MAX) {
-                    /* there was an integer overflow! */
-                    error
-                        ("Error while setting next exe time for job '%s': cl_nextexe"
-                         " overflowed (case1). now=%lu, cl_timefreq=%lu, cl_nextexe=%lu.",
-                         cl->cl_shell, now, cl->cl_timefreq, cl->cl_nextexe);
-                    error
-                        ("Setting cl_nextexe to TIME_T_MAX to prevent an infinite loop.");
-                    cl->cl_nextexe = TIME_T_MAX;
-                }
-            }
-        }
-        else {
-            if (cl->cl_nextexe != LONG_MAX) {
-                cl->cl_nextexe += slept;
-                if (cl->cl_nextexe < now || cl->cl_nextexe > TIME_T_MAX) {
-                    /* either there was an integer overflow, or the slept time is incorrect
-                     * (e.g. fcron didn't shut down cleanly and the fcrontab wasn't saved correctly) */
-                    error
-                        ("Error while setting next exe time for job '%s': cl_nextexe"
-                         " overflowed (case2). now=%lu, cl_timefreq=%lu, cl_nextexe=%lu. "
-                         "Did fcron shut down cleanly?",
-                         cl->cl_shell, now, cl->cl_timefreq, cl->cl_nextexe);
-                    error
-                        ("Setting cl_nextexe to now+cl_timefreq to prevent an infinite loop.");
-                    cl->cl_nextexe = now + cl->cl_timefreq;
-                    error("next execution will now be at %ld.", cl->cl_nextexe);
-                }
-            }
-        }
-
-        insert_nextexe(cl);
-    }
-
-    if (debug_opt && !(is_runonce(cl->cl_option) && is_hasrun(cl->cl_option))) {
-        struct tm *ftime;
-        ftime = localtime(&(cl->cl_nextexe));
-        debug("  cmd '%s' next exec %04d-%02d-%02d wday:%d %02d:%02d:%02d"
-              " (system time)",
-              cl->cl_shell, (ftime->tm_year + 1900), (ftime->tm_mon + 1),
-              ftime->tm_mday, ftime->tm_wday, ftime->tm_hour, ftime->tm_min,
-              ftime->tm_sec);
-    }
+    set_next_exe_startup(cl, is_system_startup ? CONTEXT_BOOT : CONTEXT_DEFAULT,
+                         sleep_duration);
 
     /* add the current line to the list, and allocate a new line */
     if ((cl->cl_id = next_id++) >= ULONG_MAX - 1) {
@@ -1040,8 +898,6 @@ delete_file(const char *user_name)
     cf_t *prev_file = NULL;
     cl_t *line;
     cl_t *cur_line;
-    struct job_t *j = NULL;
-    struct job_t *prev_j;
     int i, k;
     struct cl_t **s_a = NULL;
     exe_t *e = NULL;
@@ -1123,18 +979,7 @@ delete_file(const char *user_name)
             cur_line = line->cl_next;
 
             /* remove from the main queue */
-            prev_j = NULL;
-            for (j = queue_base; j != NULL; j = j->j_next)
-                if (j->j_line == line) {
-                    if (prev_j != NULL)
-                        prev_j->j_next = j->j_next;
-                    else
-                        queue_base = j->j_next;
-                    Free_safe(j);
-                    break;
-                }
-                else
-                    prev_j = j;
+            job_queue_remove(line);
 
             /* free line itself */
             free_line(line);
