@@ -1,7 +1,7 @@
 /*
  * FCRON - periodic command scheduler 
  *
- *  Copyright 2000-2016 Thibault Godouet <fcron@free.fr>
+ *  Copyright 2000-2021 Thibault Godouet <fcron@free.fr>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -39,6 +39,8 @@ void move_time_to(int where, cl_t * line, struct tm *ftime);
 void run_serial_job(void);
 void run_lavg_job(lavg_t * l);
 void run_queue_job(cl_t * line);
+int is_loop_in_queue(void);
+
 
 void
 test_jobs(void)
@@ -246,6 +248,29 @@ run_queue_job(cl_t * line)
 
 }
 
+int
+is_loop_in_queue(void)
+    /* Check that we don't have a loop in the queue_base list.
+     * Return 1 if there is a loop, 0 otherwise.
+     * Only intended to be used when running in debug mode really. */
+{
+    const int max_entries = 1000000 ;
+    int i = 0;
+    struct job_t *j;
+
+    if (queue_base == NULL) {
+        return 0;
+    }
+
+    for (j = queue_base, i = 0; j != NULL; j = j->j_next, i++) {
+        if (i > max_entries) {
+            return 1;
+        }
+    }
+
+    return 0;
+}
+
 job_t *
 job_queue_remove(cl_t * line)
     /* remove a job from the queue list
@@ -258,6 +283,12 @@ job_queue_remove(cl_t * line)
     if (queue_base == NULL)
         return NULL;
 
+    if (debug_opt && is_loop_in_queue()) {
+        error("Loop found in job queue before removing line '%s': aborting",
+              line->cl_shell);
+        abort();
+    }
+
     /* find the job in the list */
     for (j = queue_base; j != NULL; jprev = j, j = j->j_next) {
         if (j->j_line == line) {
@@ -270,6 +301,13 @@ job_queue_remove(cl_t * line)
                 queue_base = j->j_next;
 
             Free_safe(j);
+
+            if (debug_opt && is_loop_in_queue()) {
+                error("Loop found in job queue after removing line '%s': aborting.",
+                      line->cl_shell);
+                abort();
+            }
+
             return jprev;
         }
     }
@@ -286,6 +324,12 @@ insert_nextexe(cl_t * line)
     struct job_t *j = NULL;
     struct job_t *jprev = NULL;
 
+    if (debug_opt && is_loop_in_queue()) {
+        error("Detected loop in queue_base before adding job '%s' to the queue: aborting.",
+              line->cl_shell);
+        abort();
+    }
+
     Alloc(newjob, job_t);
     newjob->j_line = line;
     newjob->j_next = NULL;
@@ -297,7 +341,6 @@ insert_nextexe(cl_t * line)
     }
 
     jprev = job_queue_remove(line);
-    j = (jprev) ? jprev : queue_base;
 
     /* check if we should start from queue_base or from jprev
      * (in some cases, e.g. fcrontab has just been edited, the line should
@@ -305,6 +348,9 @@ insert_nextexe(cl_t * line)
     if (jprev == NULL || line->cl_nextexe < jprev->j_line->cl_nextexe) {
         j = queue_base;
     }
+    else {
+        j = jprev;
+    }
 
     /* a job can only be moved back */
     while (j != NULL && (line->cl_nextexe >= j->j_line->cl_nextexe)) {
@@ -315,10 +361,18 @@ insert_nextexe(cl_t * line)
 
     newjob->j_next = j;
 
-    if (jprev == NULL)
+    if (jprev == NULL) {
         queue_base = newjob;
-    else
+    }
+    else {
         jprev->j_next = newjob;
+    }
+
+    if (debug_opt && is_loop_in_queue()) {
+        error("Detected loop in queue_base after adding job '%s' to the queue: aborting.",
+               line->cl_shell);
+        abort();
+    }
 
 }
 
@@ -1274,7 +1328,6 @@ set_next_exe(cl_t * line, char option, int info_fd)
 
 }
 
-
 void
 set_next_exe_notrun(cl_t * line, char context)
     /* set the time of the next execution and send a mail to tell user his job
@@ -1336,6 +1389,222 @@ set_next_exe_notrun(cl_t * line, char context)
 
 }
 
+void
+reschedule_all_on_resume(const time_t sleep_duration)
+/* walk through all files and lines, update the schedule and run as appropriate */
+{
+    cf_t *file = NULL;
+
+    for (file = file_base; file; file = file->cf_next) {
+        cl_t *line = NULL;
+
+        debug("Re-scheduling %s's jobs...", file->cf_user);
+
+        for (line = file->cf_line_base; line; line = line->cl_next) {
+            set_next_exe_startup(line, CONTEXT_RESUME, sleep_duration);
+        }
+
+    }
+}
+
+void
+set_next_exe_startup(struct cl_t *cl, const int context,
+                     const time_t sleep_duration)
+    /* Schedule the next execution at startup (or a new file,
+     * or after a computer suspend/hibernation */
+{
+    int is_new_file = (sleep_duration == now) ? 1 : 0;
+
+    /* if job was stopped during execution: insert it in lavg or serial queue
+     * if it was in one when fcron stopped.
+     * This only applies to fcron startup and not system resume, as in the latter case
+     * the job would still be running in the background: in that case we leave it
+     * to finish normally and we don't run them again. */
+    /* NOTE:
+     * - runatreboot has higher priority than jobs that were still running
+     *   when fcron stopped, because the former will get run quicker as they are not
+     *   put into the serial queue. runatreboot jobs will be handled later on. */
+    if (context != CONTEXT_RESUME && cl->cl_numexe > 0
+        && !is_runatreboot(cl->cl_option)) {
+
+        cl->cl_numexe = 0;
+        if (is_lavg(cl->cl_option)) {
+            if (!is_strict(cl->cl_option))
+                add_lavg_job(cl, -1);
+        }
+        else if (is_serial(cl->cl_option)
+                 || is_serial_once(cl->cl_option))
+            add_serial_job(cl, -1);
+        else {
+            /* job has been stopped during execution :
+             * launch it again */
+            warn("job '%s' did not finish : running it again.", cl->cl_shell);
+            set_serial_once(cl->cl_option);
+            add_serial_job(cl, -1);
+        }
+    }
+
+    if (context == CONTEXT_BOOT
+        || (context == CONTEXT_DEFAULT && is_volatile(cl->cl_option))
+        || (context == CONTEXT_RESUME && is_runatresume(cl->cl_option))) {
+        clear_hasrun(cl->cl_option);
+    }
+
+    if (is_runonce(cl->cl_option) && is_hasrun(cl->cl_option)) {
+        /* if we get here, then context != CONTEXT_BOOT and_volatile is false */
+        /* do nothing: don't re-schedule or add to the job queue */
+        explain("job '%s' with runonce set has already run since last "
+                "system startup: not re-scheduling.", cl->cl_shell);
+    }
+    else if (is_td(cl->cl_option)) {
+
+        /* set the time and date of the next execution  */
+        if ((context == CONTEXT_BOOT && is_runatreboot(cl->cl_option))
+            || (context == CONTEXT_RESUME && is_runatresume(cl->cl_option))) {
+
+            if (is_notice_notrun(cl->cl_option)) {
+
+                if (cl->cl_runfreq == 1) {
+                    /* %-line */
+                    set_next_exe_notrun(cl, SYSDOWN_RUNATREBOOT);
+                }
+                else {
+                    /* set next exe and mail user */
+                    time_t since = cl->cl_nextexe;
+
+                    cl->cl_nextexe = now;
+                    mail_notrun_time_t(cl, SYSDOWN, since);
+                }
+
+            }
+            else {
+                cl->cl_nextexe = now;
+            }
+
+            insert_nextexe(cl);
+
+        }
+        else if (cl->cl_nextexe <= now) {
+            if (cl->cl_nextexe == 0)
+                /* the is a line from a new file */
+                set_next_exe(cl, NO_GOTO, -1);
+            else if (cl->cl_runfreq == 1 && is_notice_notrun(cl->cl_option))
+                set_next_exe_notrun(cl, SYSDOWN);
+            else if (is_bootrun(cl->cl_option) && !is_new_file
+                     && cl->cl_runfreq != 1) {
+                if (cl->cl_remain > 0 && --cl->cl_remain > 0) {
+                    debug("    cl_remain: %d", cl->cl_remain);
+                }
+                else {
+                    /* run bootrun jobs */
+                    cl->cl_remain = cl->cl_runfreq;
+                    debug("   boot-run '%s'", cl->cl_shell);
+                    if (!is_lavg(cl->cl_option)) {
+                        set_serial_once(cl->cl_option);
+                        add_serial_job(cl, -1);
+                    }
+                    else
+                        add_lavg_job(cl, -1);
+                }
+                set_next_exe(cl, STD, -1);
+            }
+            else {
+                if (is_notice_notrun(cl->cl_option)) {
+                    /* set next exe and mail user */
+                    time_t since = cl->cl_nextexe;
+
+                    set_next_exe(cl, NO_GOTO, -1);
+                    mail_notrun_time_t(cl, SYSDOWN, since);
+
+                }
+                else
+                    set_next_exe(cl, NO_GOTO, -1);
+            }
+        }
+        else {
+            /* value of nextexe is valid : just insert line in queue unless
+             * this is a system resume, in which case the line will be there
+             * already: */
+            if (context != CONTEXT_RESUME) {
+                insert_nextexe(cl);
+            }
+        }
+    }
+    else {                      /* is_td(cl->cl_option) */
+        if (cl->cl_timefreq < 1) {
+            error("Invalid timefreq %ld for job '%s': setting to 1 day",
+                  cl->cl_timefreq, cl->cl_shell);
+            cl->cl_timefreq = 3600 * 24;
+        }
+
+        /* standard @-lines */
+        if ((context == CONTEXT_BOOT && is_runatreboot(cl->cl_option))
+            || (context == CONTEXT_RESUME && is_runatresume(cl->cl_option))) {
+            cl->cl_nextexe = now;
+        }
+        else if (is_new_file || is_volatile(cl->cl_option)
+                 || (context == CONTEXT_BOOT && (is_rebootreset(cl->cl_option)
+                                                 || is_runonce(cl->cl_option)))) {
+            /* cl_first is always saved to disk for a volatile line */
+            if (cl->cl_first == LONG_MAX) {
+                cl->cl_nextexe = TIME_T_MAX;
+            }
+            else {
+                cl->cl_nextexe = now + cl->cl_first;
+                if (cl->cl_nextexe < now || cl->cl_nextexe > TIME_T_MAX) {
+                    /* there was an integer overflow! */
+                    error
+                        ("Error while setting next exe time for job '%s': cl_nextexe"
+                         " overflowed (case1). now=%lu, cl_timefreq=%lu, cl_nextexe=%lu.",
+                         cl->cl_shell, now, cl->cl_timefreq, cl->cl_nextexe);
+                    error
+                        ("Setting cl_nextexe to TIME_T_MAX to prevent an infinite loop.");
+                    cl->cl_nextexe = TIME_T_MAX;
+                }
+            }
+        }
+        else {
+            if (cl->cl_nextexe != LONG_MAX) {
+                cl->cl_nextexe += sleep_duration;
+
+                if (cl->cl_nextexe < now && context == CONTEXT_RESUME) {
+                    /* userland processes including fcron may not run for a short
+                     * time before suspend, or after resume, so we can end up in this situation.
+                     * it's not really an error though, and it's best to simply run that job now */
+                    cl->cl_nextexe = now;
+                }
+                else if (cl->cl_nextexe < now || cl->cl_nextexe > TIME_T_MAX) {
+                    /* either there was an integer overflow, or the sleep_duration time is incorrect
+                     * (e.g. fcron didn't shut down cleanly and the fcrontab wasn't saved correctly) */
+                    error
+                        ("Error while setting next exe time for job '%s': cl_nextexe"
+                         " overflowed (case2). now=%lu, cl_timefreq=%lu, cl_nextexe=%lu. "
+                         "Did fcron shut down cleanly?",
+                         cl->cl_shell, now, cl->cl_timefreq, cl->cl_nextexe);
+                    error
+                        ("Setting cl_nextexe to now+cl_timefreq to prevent an infinite loop.");
+                    cl->cl_nextexe = now + cl->cl_timefreq;
+                    error("next execution will now be at %ld.", cl->cl_nextexe);
+                }
+            }
+        }
+
+        insert_nextexe(cl);
+    }
+
+    if (debug_opt && !(is_runonce(cl->cl_option) && is_hasrun(cl->cl_option))) {
+        struct tm *ftime;
+        ftime = localtime(&(cl->cl_nextexe));
+        debug("  cmd '%s' next exec %04d-%02d-%02d wday:%d %02d:%02d:%02d"
+              " (system time)",
+              cl->cl_shell, (ftime->tm_year + 1900), (ftime->tm_mon + 1),
+              ftime->tm_mday, ftime->tm_wday, ftime->tm_hour, ftime->tm_min,
+              ftime->tm_sec);
+    }
+
+}
+
+
 void
 mail_notrun_time_t(cl_t * line, char context, time_t since_time_t)
 /* Same as mail_notrun() but with 'since' defined as a time_t instead of a struct tm */
@@ -1403,9 +1672,10 @@ mail_notrun(cl_t * line, char context, struct tm *since)
 
     switch (context) {
     case SYSDOWN:
+    case SYSDOWN_RUNATREBOOT:
         fprintf(mailf, "Line '%s' has not run since and including "
                 "%04d-%02d-%02d wday:%d %02d:%02d (timezone=%s)\n"
-                "due to system's down state.\n",
+                "due to system's down or suspended state.\n",
                 line->cl_shell, (since->tm_year + 1900), (since->tm_mon + 1),
                 since->tm_mday, since->tm_wday, since->tm_hour, since->tm_min,
                 (line->cl_tz) ? line->cl_tz : "system's");
@@ -1441,6 +1711,8 @@ mail_notrun(cl_t * line, char context, struct tm *since)
                 "serialonce and/or fcron's option -m.\n");
         fprintf(mailf, "Note that job '%s' has not run.\n", line->cl_shell);
         break;
+    default:
+        error("mail_notrun() called with unkown context '%c'. Ignoring.");
     }
 
     /* become user (for security reasons) */
